#usage parse_xml.rb ../../src/kegg/xml/kgml/metabolic/ ../data.sqlite

require 'rubygems'
require 'rexml/document'
require 'sqlite3'

class Maplink
  attr_reader :kos, :map, :substrates, :products, :thisMap
  def initialize(kos,map,substrates,products,thisMap)
    @kos=kos; @map=map; @substrates=substrates; @products=products; @thisMap=thisMap;
  end
end
mapLinks = []

class Pair
  attr_reader :kos1, :cpds, :kos2, :kos1Map, :kos2Map, :direction
  def initialize(kos1,cpds,kos2,kos1Map,kos2Map,direction)
    @kos1=kos1;@cpds=cpds;@kos2=kos2;@kos1Map=kos1Map;@kos2Map=kos2Map;@direction=direction;
  end
end
pairs = []

class Reaction
  attr_reader :id, :kos, :names, :type, :substrates, :products
  def initialize(id,kos,names,type,substrates,products)
    @id = id; @kos = kos; @names = names; @type = type; @substrates = []; @products = [];
    @substrates.concat(substrates)
    @products.concat(products)
    if type == 'reversible'
      @substrates.concat(products)
      @products.concat(substrates)
    end
  end
end
reactions = {}

class Compound
  attr_reader :id, :name, :substrate_of, :product_of
  attr_writer :substrate_of, :product_of
  def initialize(id,name)
    @id = id; @name = name; @substrate_of = []; @product_of = [];
  end
  def add_product_of(list)
    @product_of.concat(list) unless list.nil?
  end
  def add_substrate_of(list)
    @substrate_of.concat(list) unless list.nil?
  end
end
compoundList = {}

Dir::glob(ARGV[0]+'ko/*.xml').each {|ko_file|
  dirs = ko_file.split('/')
  mapId = dirs.last.gsub(/\.xml$/,'').gsub(/^ko/,'')
  next if mapId.to_i > 1060
  p mapId
  tmp_kos = {}
  tmp_mapId = {}
  tmp_compounds = {}
  tmp_reaction = {}
  fp = File.open(ko_file)
    doc = REXML::Document.new fp
    doc.elements.each('pathway/entry') do |entry|
      koid = entry.attributes['id']
      id = mapId+'__'+koid
      if entry.attributes['type'] == 'map'
        tmp_mapId[id] = entry.attributes['name']
      elsif entry.attributes['type'] == 'ortholog'
        next if entry.attributes['reaction'].nil?
        kos = entry.attributes['name'].split("\s").map{|ko| ko.gsub(/ko\:/, koid + '--')}
        tmp_kos[id] = kos
        tmp_reaction[id] = entry.attributes['reaction'].split("\s").map{|rn| rn.gsub(/rn\:/, '')}
      elsif entry.attributes['type'] == 'compound'
        tmp_compounds[id] = entry.attributes['name'].gsub(/cpd\:|gl\:|dr\:/, '')
      end
    end
    doc.elements.each('pathway/reaction') do |reaction|
      id = "#{mapId}__#{reaction.attributes['id']}"
      names = reaction.attributes['name'].gsub(/rn\:/, '').split("\s")
      type = reaction.attributes['type']
      substrates = []
      products = []
      reaction.elements.each('substrate') do |substrate|
        name = substrate.attributes['name'].gsub(/cpd\:|gl\:|dr\:/, '')
        substrates.push(name)
        compoundList["#{mapId}__#{name}"] = Compound.new("#{mapId}__#{name}", name) if compoundList["#{mapId}__#{name}"].nil?
        compoundList["#{mapId}__#{name}"].add_substrate_of(tmp_kos[id])
        compoundList["#{mapId}__#{name}"].add_product_of(tmp_kos[id]) if type == 'reversible'
      end
      reaction.elements.each('product') do |product|
        name = product.attributes['name'].gsub(/cpd\:|gl\:|dr\:/, '')
        products.push(name)
        compoundList["#{mapId}__#{name}"] = Compound.new("#{mapId}__#{name}", name) if compoundList["#{mapId}__#{name}"].nil?
        compoundList["#{mapId}__#{name}"].add_product_of(tmp_kos[id])
        compoundList["#{mapId}__#{name}"].add_substrate_of(tmp_kos[id]) if type == 'reversible'
      end
      reactions[id] = Reaction.new(id, tmp_kos[id], names, type, substrates, products)
    end
    doc.elements.each('pathway/relation') do |relation|
      compounds = []
      relation.elements.each('subtype') do |subtype|
        cpd_id = "#{mapId}__#{subtype.attributes['value']}"
        compounds.concat(tmp_compounds[cpd_id].split("\s")) unless tmp_compounds[cpd_id].nil?
      end
      if relation.attributes['type'] == 'ECrel'
        ko1_id = "#{mapId}__#{relation.attributes['entry1']}"
        ko2_id = "#{mapId}__#{relation.attributes['entry2']}"
        next if tmp_kos[ko1_id].nil? || tmp_kos[ko2_id].nil? || reactions[ko1_id].nil? || reactions[ko2_id].nil?
        ko1_ko2_cpds = reactions[ko1_id].products & reactions[ko2_id].substrates & compounds
        if ko1_ko2_cpds.length > 0
          pairs.push(Pair.new(tmp_kos[ko1_id], ko1_ko2_cpds.join(','), tmp_kos[ko2_id], mapId, mapId, '1->c->2'))
        end
        ko2_ko1_cpds = reactions[ko2_id].products & reactions[ko1_id].substrates & compounds
        if ko2_ko1_cpds.length > 0
          pairs.push(Pair.new(tmp_kos[ko1_id], ko2_ko1_cpds.join(','), tmp_kos[ko2_id], mapId, mapId, '1<-c<-2'))
        end
        to_cpds = reactions[ko1_id].products & reactions[ko2_id].products & compounds
        if to_cpds.length > 0
          pairs.push(Pair.new(tmp_kos[ko1_id], to_cpds.join(','), tmp_kos[ko2_id], mapId, mapId, '1->c<-2'))
        end
        from_cpds = reactions[ko1_id].substrates & reactions[ko2_id].substrates & compounds
        if from_cpds.length > 0
          pairs.push(Pair.new(tmp_kos[ko1_id], from_cpds.join(','), tmp_kos[ko2_id], mapId,mapId, '1<-c->2'))
        end
      elsif relation.attributes['type'] == 'maplink'
        id1 = "#{mapId}__#{relation.attributes['entry1']}"
        id2 = "#{mapId}__#{relation.attributes['entry2']}"
        ko_id = tmp_kos[id1].nil? ? id2 : id1
        map_id = tmp_mapId[id1].nil? ? id2 : id1
        next if tmp_kos[ko_id].nil? || tmp_mapId[map_id].nil? || reactions[ko_id].nil?
        products = reactions[ko_id].products & compounds
        substrates = reactions[ko_id].substrates & compounds
        mapLinks.push(Maplink.new(tmp_kos[ko_id], tmp_mapId[map_id], substrates, products, mapId))
      end
    end
  fp.close
}

mapLinks.each do |mapLink|
  map = mapLink.map.gsub('path:ko', '')
  mapLink.substrates.each do |cpd|
    next if compoundList["#{map}__#{cpd}"].nil?
    ko_inMap = compoundList["#{map}__#{cpd}"].product_of
    pairs.push(Pair.new(ko_inMap, cpd, mapLink.kos, map, mapLink.thisMap, '1->c->2')) unless ko_inMap.nil?
    ko_inMap = compoundList["#{map}__#{cpd}"].substrate_of
    pairs.push(Pair.new(ko_inMap, cpd, mapLink.kos, map, mapLink.thisMap, '1<-c->2')) unless ko_inMap.nil?
  end
  mapLink.products.each do |cpd|
    next if compoundList["#{map}__#{cpd}"].nil?
    ko_inMap = compoundList["#{map}__#{cpd}"].substrate_of
    pairs.push(Pair.new(mapLink.kos, cpd, ko_inMap, mapLink.thisMap, map, '1->c->2')) unless ko_inMap.nil?
    ko_inMap = compoundList["#{map}__#{cpd}"].product_of
    pairs.push(Pair.new(mapLink.kos, cpd, ko_inMap, mapLink.thisMap, map, '1<-c->2')) unless ko_inMap.nil?
  end
end


db = SQLite3::Database.new(ARGV[1])
db.transaction do
  pairs.each do |pair|
    pair.kos1.each do |ko1|
      pair.kos2.each do |ko2|
        ko1id,ko1num = ko1.split('--')
        ko2id,ko2num = ko2.split('--')
        sql = "insert into ecrel values (NULL,'#{ko1num}','#{pair.kos1Map}','#{ko1id}','#{reactions[pair.kos1Map + '__' + ko1id].names.join(',')}','#{ko2num}','#{pair.kos2Map}','#{ko2id}','#{reactions[pair.kos2Map+'__'+ko2id].names.join(',')}','#{pair.cpds}','#{pair.direction}')"
        #p sql
        db.execute(sql)
      end
    end
  end
end #sqlite3 transaction END
db.close


#CREATE TABLE ecrel (id INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT, koid_1 TEXT, map_1 TEXT, xmlid_1 TEXT, reactions_1 TEXT, koid_2 TEXT, map_2 TEXT, xmlid_2 TEXT, reactions_2 TEXT, compounds, direction);
